package com.xhkjedu.sstudy.utils;

import com.xhkjedu.sstudy.config.ConfigKey;
import com.xhkjedu.sstudy.mapper.paper.PaperAnalyzeMapper;
import com.xhkjedu.sstudy.vo.paperstudent.PaperStudentQuestionVo;
import com.xhkjedu.sstudy.vo.paperstudent.StudentPaperTypeVo;
import com.xhkjedu.utils.N_Utils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static com.xhkjedu.utils.N_Utils.getSecondTimestamp;

/**
 * @ClassName StudyUtil
 * Description study模块用到的工具类
 * Author WN
 * Date 2022/2/21 10:01
 **/
@Slf4j
public class StudyUtil {
    public static Integer id = 0;
    public static Map<String, String> uriMap = new ConcurrentHashMap<>();

    //获取id
    public static String getId(){
        Integer nummin = ConfigKey.nummin;
        Integer nummax = ConfigKey.nummax;
        if (id == 0 || id.equals(nummax)) {
            id = nummin;
        } else {
            id ++;
        }
        String _num = (getSecondTimestamp() + ConfigKey.numadd) +"" + id;
        long num = Long.parseLong(_num);
        return Long.toString(num,36).toUpperCase();
    }

    /**
     * 发送form-data请求返回json结果
     * @Param [apiUrl, param]
     * @Author ywx
     * @Date 2021/5/14 17:41
     * @return java.lang.String
     **/
    public static String sendFormData(String apiUrl, MultiValueMap<String, String> param) {
        try {
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
            List<MediaType> acceptableMediaTypes = new ArrayList<>();
            acceptableMediaTypes.add(MediaType.APPLICATION_FORM_URLENCODED);
            acceptableMediaTypes.add(MediaType.APPLICATION_PROBLEM_JSON_UTF8);
            HttpEntity<MultiValueMap<String, String>> requestParams = new HttpEntity<>(param, headers);
            ResponseEntity<String> response = new RestTemplate().postForEntity(apiUrl, requestParams, String.class);
            String result = response.getBody();
            return result;
        } catch (RestClientException e) {
            log.error("发送form-data请求失败",e.getMessage());
            return null;
        }
    }

    //处理母子题
    public static List<StudentPaperTypeVo> setPaperQuestion(List<StudentPaperTypeVo> types, PaperAnalyzeMapper paperAnalyzeMapper){
        List<PaperStudentQuestionVo> qlist = new ArrayList<>();
        types.forEach(t -> {
            qlist.addAll(t.getQuestions());
        });
        List<String> questionpids = qlist.stream().filter(q -> q.getQlevel() == 3)
                .map(q -> q.getQuestionpid()).distinct().collect(Collectors.toList());
        List<PaperStudentQuestionVo> mquestions = new ArrayList<>();
        if (N_Utils.isListNotEmpty(questionpids)) {
            mquestions = paperAnalyzeMapper.listQuestionByQuestionIds(questionpids);
        }
        for (StudentPaperTypeVo t : types) {
            List<PaperStudentQuestionVo> questionList = new ArrayList<>();
            List<PaperStudentQuestionVo> questions = t.getQuestions();
            LinkedHashMap<String, List<PaperStudentQuestionVo>> collect = questions.stream()
                    .collect(Collectors.groupingBy(q -> q.getQuestionpid(), LinkedHashMap::new, Collectors.toList()));
            for (Map.Entry<String, List<PaperStudentQuestionVo>> entry : collect.entrySet()) {
                String questionpid = entry.getKey();
                List<PaperStudentQuestionVo> list = entry.getValue();
                PaperStudentQuestionVo tq = list.get(0);
                PaperStudentQuestionVo q;
                if(tq.getQlevel()==3){//子题
                    q = mquestions.stream().filter(mq -> mq.getQuestionid().equals(questionpid)).findFirst().orElse(null);
                    double score = list.stream().mapToDouble(PaperStudentQuestionVo::getQscore).sum();
                    List<PaperStudentQuestionVo> sonques = list.stream().filter(l -> l.getQuestionpid().equals(questionpid)).collect(Collectors.toList());
                    q.setQscore(score);
                    q.setQorder(tq.getQorder());
                    q.setSonques(sonques);
                }else {
                    q=tq;
                }
                questionList.add(q);
            }
            t.setQuestions(questionList);
        }
        return types;
    }

    /**
     * @Description 分值平均后得到的值，如果整除返回数组元素1，未整除返回元素数量2
     * @Param [num, score] 数量，分值
     * @Return java.lang.Double[] 第一个未平均分值，第二个值为最后一个数的分值
     * @Author wn
     * @Date 2022/8/18 11:07
     **/
    public static Double[] getPointAvgScore(int num, Double score) {
        BigDecimal bScore = new BigDecimal(Double.toString(score));
        BigDecimal bNum = new BigDecimal(Integer.toString(num));

        BigDecimal avg = bScore.divide(bNum, 1, BigDecimal.ROUND_DOWN);
        BigDecimal avgAllScore = avg.multiply(bNum);//平均分乘以数量得到的分值
        if (bScore.equals(avgAllScore)) {
            //说明整除
            return new Double[]{avg.doubleValue()};
        } else {
            //未整除，计算最后剩余分值，subtract总分减去平均分和数量的积
            // 平均分加上剩余得分得到最后一个分值
            BigDecimal lastScore = avg.add(bScore.subtract(avgAllScore));
            return new Double[]{avg.doubleValue(), lastScore.doubleValue()};
        }
    }
}
